/* Copyright (C) Sportradar AG. See LICENSE for full license governing this code */
package com.sportradar.unifiedodds.restfuloddscache;

import java.io.File;
import java.io.PrintWriter;
import java.lang.management.ManagementFactory;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.server.handler.ResourceHandler;
import org.json.JSONObject;
import org.json.XML;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sportradar.unifiedodds.restfuloddscache.oddsviewer.OddsViewerLogic;
import com.sportradar.unifiedodds.restfuloddscache.oddsviewer.OddsViewerWebsocketHandler;
import com.sportradar.unifiedodds.sdk.MessageInterest;
import com.sportradar.unifiedodds.sdk.OddsFeedConfiguration;
import com.sportradar.unifiedodds.sdk.OddsFeedSession;
import com.sportradar.unifiedodds.sdk.OddsFeedSessionBuilder;
import com.sportradar.unifiedodds.sdk.SDKGlobalEventsListener;
import com.sportradar.unifiedodds.sdk.oddsentities.ProductDown;
import com.sportradar.unifiedodds.sdk.sportentities.SportEvent;
import com.sportradar.utils.Cache;

/**
 * Main class for the Restful OddsCache -Duf.accesstoken=<access-token> needs to be set to start
 */
public class RestfulOddsCacheMain {

	private static class GlobalEventsListener implements SDKGlobalEventsListener {

		@Override
		public void onProductDown(ProductDown productDown) {

		}

		@Override
		public void onConnectionDown() {
			System.out.println("Connection down");
		}
	}

	private static OddsCacheStats statsBean;

	public static void main(String[] args) throws Exception {

		OddsFeedConfiguration config = new OddsFeedConfiguration();

		String mqHost = System.getProperty("uf.mqhost");
		if (mqHost != null) {
			config.setMessagingHost(mqHost);
		}

		String apiHost = System.getProperty("uf.apihost");
		if (apiHost != null) {
			config.setAPIHost(apiHost);
		}

		// currently we say we are new and have nothing (uncomment the below to do a recovery
		// as if we have been out for 5 minutes)
		//config.setLastReceivedTimestampBeforeDisconnect(System.currentTimeMillis() - 1 * 60 * 1000L);
		OddsFeedSessionBuilder builder = OddsFeedSessionBuilder.init(new GlobalEventsListener(), config);
		Cache<String, List<byte[]>> lm = new Cache<>("Logs", 60 * 60);
		OddsListener listener = new OddsListener(lm);
		MBeanServer mbeanServer = ManagementFactory.getPlatformMBeanServer();
		try {
			ObjectName name = new ObjectName("com.sportradar.unifiedodds:type=RestfulOddsCache");
			statsBean = new OddsCacheStats(listener, lm);
			mbeanServer.registerMBean(statsBean, name);
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		OddsFeedSession session = builder.setListener(listener).setMessageInterest(MessageInterest.AllMessages).build();

		session.waitForInitialSnapshotToComplete();

		HandlerList handlers = new HandlerList();

		HTTPOddsRequestHandler handler = new HTTPOddsRequestHandler(listener, session.getMarketDescriptionManager(), session.getSportsInfoManager());

		OddsViewerWebsocketHandler webSocketHandler = new OddsViewerWebsocketHandler();

		String webDir = RestfulOddsCacheMain.class.getClassLoader().getResource("www/").toExternalForm();
		ResourceHandler resourceHandler = new ResourceHandler();
		resourceHandler.setDirectoriesListed(false);
		resourceHandler.setResourceBase(webDir);

		handlers.addHandler(resourceHandler);
		handlers.addHandler(handler);
		handlers.addHandler(webSocketHandler);

		Server server = new Server(8015);
		server.setHandler(handlers);
		server.start();

		PrintWriter pw = new PrintWriter(new File("all_changes.txt"));
		final Logger logger = LoggerFactory.getLogger("roc");
		final ScheduledExecutorService timer = Executors.newSingleThreadScheduledExecutor();
		timer.scheduleWithFixedDelay(new Runnable() {
			@Override
			public void run() {
				logger.debug("Clearing unused logs");
				lm.purge();
			}
		}, 1, 10, TimeUnit.MINUTES);

		while (true) {
			MatchOdds mo;
			while ((mo = listener.getNextChange()) != null) {
				LinkedList<MarketOdds> lsi = null;
				SportEvent event;
				synchronized (mo) {
					event = mo.getSportEvent();
					lsi = new LinkedList<MarketOdds>(mo.getMarkets());
					try {
						if (OddsViewerLogic.getInstance().isEventViewed(event.getId())) {
							StringBuilder xml = handler.getOddsResponse(mo, false);
							JSONObject xmlJSONObj = XML.toJSONObject(xml.toString());
							OddsViewerLogic.getInstance().sendMatchOdds(event.getId(), xmlJSONObj);
						}
					}
					catch (Exception ex) {
						logger.warn("failed to send match on websocket " + event.getId(), ex);
					}
				}
				pw.println(event);
				for (MarketOdds mmo : lsi) {
					handler.printMarket(pw, System.currentTimeMillis(), mmo, event, true);
				}

			}
			Thread.sleep(1000);
		}
	}
}
